/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <nostalgia/core/core.hpp>
#include <nostalgia/foundation/media.hpp>

using namespace nostalgia;

static int spriteX = 72;
static int spriteY = 64;
static ox::StringView sprites = "nostalgia";
static bool paused = false;

static int updateHandler(core::Context *ctx) noexcept {
	constexpr auto sleepTime = 16;
	if (paused) {
		return sleepTime;
	}
	int xmod = 0;
	int ymod = 0;
	if (buttonDown(ctx, core::Alpha_D) || buttonDown(ctx, core::GamePad_Right)) {
		xmod = 2;
	} else if (buttonDown(ctx, core::Alpha_A) || buttonDown(ctx, core::GamePad_Left)) {
		xmod = -2;
	}
	if (buttonDown(ctx, core::Alpha_S) || buttonDown(ctx, core::GamePad_Down)) {
		ymod = 2;
	} else if (buttonDown(ctx, core::Alpha_W) || buttonDown(ctx, core::GamePad_Up)) {
		ymod = -2;
	}
	if (!xmod && !ymod) {
		spriteX += 1;
	}
	spriteX += xmod;
	spriteY += ymod;
	//constexpr ox::StringView sprites = "nostalgia";
	for (unsigned i = 0; i < sprites.len(); ++i) {
		const auto c = static_cast<unsigned>(sprites[i] - ('a' - 1));
		core::setSprite(ctx, i, spriteX + 8 * (static_cast<int>(i) + 1), spriteY, c, 0, 0, 0);
	}
	return sleepTime;
}

static void keyEventHandler(core::Context *ctx, core::Key key, bool down) noexcept {
	if (down) {
		if (key == core::Key::Alpha_Q) {
			core::shutdown(ctx);
		} else if (key == core::Key::Alpha_P) {
			paused = !paused;
		}
	}
}

ox::Error run(ox::UniquePtr<ox::FileSystem> fs) noexcept {
	oxTraceInitHook();
	oxRequireM(ctx, core::init(std::move(fs)));
	constexpr ox::FileAddress TileSheetAddr("/TileSheets/Charset.ng");
	constexpr ox::FileAddress PaletteAddr("/Palettes/Charset.npal");
	oxRequire(tsStat, ctx->rom->stat(PaletteAddr));
	oxReturnError(core::loadSpriteTileSheet(ctx.get(), TileSheetAddr, PaletteAddr));
	oxReturnError(core::initConsole(ctx.get()));
	core::puts(ctx.get(), 10, 9, "DOPENESS!!!");
	core::setUpdateHandler(ctx.get(), updateHandler);
	core::setKeyEventHandler(ctx.get(), keyEventHandler);
	return core::run(ctx.get());
}
